/*
 * @(#) $Id: $
 * 
 * Copyright 2009/2010 by sIT Solutions, A-1110 Wien, Geiselbergstr.21-25. All rights reserved.
 */
package org.openxma.dsl.core.linking.support;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * String builder that builds description strings by accepting javaDoc style commented text strings in DSL files
 */

public class JavaDocStringBuilder {
    public static final String NEWLINE = "\n";
    private static final String JAVADOC_END = "*/";
    private static final String JAVADOC_START = "/**";
    private StringBuilder javaDocLines = new StringBuilder();

    private static final int STATUS_IDLE = 1;
    private static final int STATUS_BUILDING = 2;
    private static final int STATUS_COMPLETE = 3;

    private int status = STATUS_IDLE;

    private static final Pattern LEADING_STARS_PATTERN = Pattern.compile("^(\\s*[*]).*");
    private static final Pattern EMPTY_LINE_PATTERN = Pattern.compile("^\\s*[*]*\\s*$");

    /**
     * Accepts text lines including the JavaDoc comment strings
     */
    public void newText(String text) {
        for (String line : text.split("\n"))
            newTextLine(line);
    }
    private void newTextLine(String text) {
        switch (status) {
        case STATUS_COMPLETE:
        case STATUS_IDLE:
            if (includesJavaDocStart(text)) {
                if (includesJavaDocEnd(text)) {
                    status = STATUS_COMPLETE;
                } else {
                    status = STATUS_BUILDING;
                }
                appendDoc(textBetween(text));
            }
            break;
        case STATUS_BUILDING:
            if (includesJavaDocEnd(text)) {
                status = STATUS_COMPLETE;
            }
            appendDoc(textBetween(text));
            break;
        }
    }

    public boolean isJavaDocCommentCompleted() {
        return status == STATUS_COMPLETE;
    }

    public String getJavaDocAndReset() {
        String returnString = javaDocLines.toString();
        reset();
        return returnString;
    }

    public void reset() {
        if (status == STATUS_IDLE)
            return;
        status = STATUS_IDLE;
        javaDocLines = new StringBuilder();
    }

    private void appendNewLine() {
        javaDocLines.append(NEWLINE);
    }

    private void appendDoc(String text) {
        text = removeEmptyLine(text);
        text = removeReturns(text);
        text = removeLeadingStars(text);
        if (status != STATUS_IDLE && !"".equals(text.trim()) && !"".equals(javaDocLines.toString().trim()))
            appendNewLine();
        javaDocLines.append(text);
    }

    private String removeLeadingStars(String text) {
        Matcher matcher = LEADING_STARS_PATTERN.matcher(text);
        if (matcher.matches()) {
            String group1 = matcher.group(1);
            text = text.substring(group1.length());
        }
        return text;
    }
    private String removeReturns(String text) {
        return text.replaceAll("\r", "");
    }

    private String removeEmptyLine(String text) {
        Matcher matcher = EMPTY_LINE_PATTERN.matcher(text);
        if (matcher.matches()) {
            return "";
        }
        return text;
    }

    private static boolean includesJavaDocStart(String text) {
        return text.contains(JAVADOC_START);
    }

    private static boolean includesJavaDocEnd(String text) {
        return text.contains(JAVADOC_END);
    }

    private static int indexOfJavaDocStart(String text) {
        return text.indexOf(JAVADOC_START);
    }

    private static int indexOfJavaDocEnd(String text) {
        return text.lastIndexOf(JAVADOC_END);
    }

    private static String textBetween(String text) {
        int s = indexOfJavaDocStart(text);
        int e = indexOfJavaDocEnd(text);
        s = (s != -1) ? s + JAVADOC_START.length() : 0;
        e = (e != -1) ? e : text.length();
        return text.substring(s, e);
    }
}
